package github.benjamin.equipreservebackend.vo;

import github.benjamin.equipreservebackend.constant.ReservationStatus;
import github.benjamin.equipreservebackend.entity.Device;
import github.benjamin.equipreservebackend.entity.Reservation;
import github.benjamin.equipreservebackend.entity.User;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.Objects;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class UserReservationVO {
    private String reservationId;
    private String deviceName;
    private LocalDate startTime;
    private LocalDate endTime;
    private String statusLabel;
    private String deviceAdminName;
    private String deviceAdminContact;
    private LocalDateTime createdTime;


    public UserReservationVO(Reservation r, Map<Long, Device> deviceMap, Map<Long, User> deviceAdminMap) {
        this.reservationId = r.getId().toString();
        this.deviceName = deviceMap.get(r.getDeviceId()).getName();
        this.startTime = r.getStartTime();
        this.endTime = r.getEndTime();
        this.statusLabel = ReservationStatus.valueOf(r.getStatus()).getLabel();
        User deviceAdmin = deviceAdminMap.get(r.getDeviceAdminId());
        if (Objects.nonNull(deviceAdmin)) {
            this.deviceAdminName = deviceAdmin.getName();
            this.deviceAdminContact = deviceAdmin.getPhone();
        } else {
            this.deviceAdminName = "该管理员不存在，请联系设备相关人员";
            this.deviceAdminContact = "";
        }
        this.createdTime = r.getCreatedTime();

    }
}
